/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _AIEASY_H
#define	_AIEASY_H

#include "aibase.h"

class WeightedCoordCell : public TCoordCell
{
	public:
		WeightedCoordCell(const Board& _board) : TCoordCell(-1, -1), board (_board) {
			value = -1;
			player = NONE_PLAYER;
		}
		WeightedCoordCell(const Board& _board, TPlayer player, const TCoordCell& _cell) : TCoordCell(_cell), board(_board) {
			this->player = player;
			value = -1;
			if (board.isValidCell(*this)) {
				// prefer cells with lower values (1 -> 4, ..., 4-> 1)
				value = 5 - board.getCellValue(*this);
				addNeighborValue(*this+TCoordCell(0, board.pointsDown(*this) ? -1 : 1));
				addNeighborValue(*this+TCoordCell(1, 0));
				addNeighborValue(*this+TCoordCell(-1, 0));
			}
		}

		int getValue() {
			return value;
		}

    bool operator < (const WeightedCoordCell& cell) const {
			if (col == -1)
				return true;
			return (value < cell.value);
    }

    WeightedCoordCell& operator = (const WeightedCoordCell& cell) {
			col = cell.col;
			row = cell.row;
			value = cell.value;
			return *this;
		}

	protected:
		const Board& board;
		TPlayer player;
		int value;

		void addNeighborValue(const TCoordCell& neighbor) {
			if (!board.isValidCell(neighbor))
				return;

			switch (board.getCellValue(neighbor)) {
				case 1:
					switch (board.getCellValue(*this)) {
						case 1: // chain reaction
							value += 2;
							break;
						case 2: // preparing chain reaction
							if (board.getCellPlayer(neighbor) == player) // for us
								value ++;
							else // for the enemy
								value -= 2;
							break;
					}
					break;
				case 2:
					if (board.getCellValue(*this) == 1) {
						TCoordCell nextNeighbor = neighbor+TCoordCell(0, board.pointsDown(neighbor) ? -1 : 1);
						if (board.isValidCell(nextNeighbor) && board.getCellValue(nextNeighbor) == 1
								&& board.getCellPlayer(nextNeighbor) != player)
							value --;
						nextNeighbor = neighbor+TCoordCell(-1, 0);
						if (board.isValidCell(nextNeighbor) && board.getCellValue(nextNeighbor) == 1
								&& board.getCellPlayer(nextNeighbor) != player)
							value --;
						nextNeighbor = neighbor+TCoordCell(1, 0);
						if (board.isValidCell(nextNeighbor) && board.getCellValue(nextNeighbor) == 1
								&& board.getCellPlayer(nextNeighbor) != player)
							value --;
					}
					break;
			}
		}
};

class AIEasy : public AIBase
{
	public:
		AIEasy(TPlayer player, const Board& board);
    virtual ~AIEasy();

		virtual TCoordCell step(int starters);

	private:
		TPlayer player;
		const Board& board;
};

#endif	/* _AIEASY_H */

