/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _BOARD_H
#define	_BOARD_H

#include <nds.h>

#include "player.h"

struct TCoordCell
{
    int col;
    int row;

    TCoordCell (const int c = 0, const int r = 0) : col (c), row (r) { }

    TCoordCell& operator = (const TCoordCell& cell) {
        col = cell.col;
        row = cell.row;
        return *this;
    }

		TCoordCell& operator + (const TCoordCell& cell) const {
			TCoordCell *sumCell = new TCoordCell();
			sumCell->row = this->row + cell.row;
			sumCell->col = this->col + cell.col;
			return *sumCell;
		}
};

class Board
{
	public:
		Board(int size);

		int getSize() const;
		int getWidth() const;
		int getHeight() const;
		bool pointsDown(const TCoordCell& c) const;
		bool pointsDown(int x, int y) const;
		bool isValidCell(const TCoordCell& c) const;
		bool isValidCell(int x, int y) const;
		u8 getCellValue(TCoordCell cell) const;
		u8 getCellValue(int x, int y) const;
		TPlayer getCellPlayer(TCoordCell cell) const;
		TPlayer getCellPlayer(int x, int y) const;
		int initValue(int x, int y) const;
		bool checkEnd(TPlayer player) const;
		bool checkLost(const Player& player) const;

		void setCellValue(TCoordCell cell, u8);
		void setCellValue(int x, int y, u8);
		void setCellPlayer(TCoordCell cell, TPlayer player);
		void setCellPlayer(int x, int y, TPlayer player);
		TCoordCell navigate(const TCoordCell& cc, const TCoordCell& dc);

		// maps cells in or next to the board to cells in the board
		TCoordCell map(const TCoordCell& cell);

	private:
		u8 *board;
		int size;
		int width;
		int height;
};

#endif	/* _BOARD_H */

