/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GAME_H
#define	_GAME_H

#include "board.h"
#include "player.h"
#include "ui.h"
#include "options.h"
#include "aibase.h"
#include "aieasy.h"

enum TGameState
{
    END_STATE,
    PLAY_STATE,
		PAUSE_STATE
};

class CellBuffer
{
	private:
		TCoordCell* buffer[100];
		int size, first, next;
	
	public:
		CellBuffer() {
			first = next = 0;
			for (int i = 0; i < 100; i++)
				buffer[i] = 0;
		}

		void add(const TCoordCell& cell) {
			buffer[next] = new TCoordCell(cell.col, cell.row);
			next = (next+1) % 100;
		}

		TCoordCell remove() {
			TCoordCell cell = *buffer[first];
			delete buffer[first];
			buffer[first] = 0;
			first = (first+1) % 100;
			return cell;
		}

		bool isEmpty() {
			return (first == next && buffer[first] == 0);
		}
		
		bool isFull() {
			return (first == next && buffer[first] != 0);
		}
};

class Game
{
	private:
		Board* board;
		UI* ui;
		Options* options;
		Player *players[3];
		CellBuffer *buffer;
		int curPlayer;
		int skip;
		TGameState state;
		bool checkEnd();
		AIEasy *ai;
		inline void addTask(const TCoordCell& cell);
		Player *getCurrentPlayer();
		void nextPlayer();

	public:
		Game(UI* ui, Options* options);
		~Game();
		const Board& getBoard();
		void turn(const TCoordCell& cell);
		void startAI();
		void setCell(TCoordCell cell, TPlayer player);
		void resetCell(TCoordCell cell, TPlayer player);
		void newColor();
		bool humansTurn();
		void onQuarterSecond();
		void toggleHelp();
};


#endif	/* _GAME_H */

