/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _UI_H
#define	_UI_H

#include <nds.h>

#include "board.h"
#include "audio.h"
#include "tiles.h"

static u16 mTileTopPetal[4*3] = {
	55|(1<<10), 42|(1<<10), 55|(1<<10), 42|(1<<10),
	38|(1<<10), 41|(3<<10), 38|(1<<10), 41|(3<<10),
	52        , 52|(1<<10), 52        , 52|(1<<10)
};

static u16 mTileBottomPetal[4*3] = {
	41        , 41|(1<<10), 41        , 41|(1<<10),
	41        , 38|(2<<10), 41        , 38|(2<<10),
	42|(2<<10), 55|(2<<10), 42|(2<<10), 55|(2<<10)
};

static u16 mTileLeft1Petal[4*4] = {
	-1        , 47|(2<<10), 53|(2<<10), 43|(1<<10),
	44|(2<<10), 46|(2<<10), 45|(1<<10), 49|(3<<10),
	39|(2<<10), 54|(2<<10), 51|(3<<10), 50|(3<<10),
	-1        , -1        , 48|(3<<10), -1        
};

static u16 mTileLeft2Petal[4*4] = {
	-1        , -1        , 48|(1<<10), -1        , 
	39        , 54        , 51|(1<<10), 50|(1<<10),
	44        , 46        , 45|(3<<10), 49|(1<<10),
	-1        , 47        , 53        , 43|(3<<10)
};

static u16 mTileRight1Petal[4*4] = {
	43        , 53|(2<<10), 47|(3<<10), -1        ,
	49|(2<<10), 45        , 46|(3<<10), 44|(3<<10),
	50|(2<<10), 51|(2<<10), 54|(3<<10), 39|(3<<10),
	-1        , 48|(2<<10), -1        , -1        
};

static u16 mTileRight2Petal[4*4] = {
	-1        , 48        , -1        , -1        ,
	50        , 51        , 54|(1<<10), 39|(1<<10),
	49        , 45|(2<<10), 46|(1<<10), 44|(1<<10),
	43|(2<<10), 53        , 47|(1<<10), -1
};

static u16 mTileDown[4*3] = {
	16|(2<<10),  0        ,  0        , 16|(3<<10),
	24|(2<<10), 20|(2<<10), 20|(3<<10), 24|(3<<10),
  -1        ,  8|(2<<10),  8|(3<<10), -1
};
static u16 mTileUp[4*3] = {
	-1,          8        ,  8|(1<<10), -1        ,
	24        , 20        , 20|(1<<10), 24|(1<<10),
  12        ,  4        ,  4        , 12|(1<<10)
};

class UI {
	public:
		UI(Audio *audio);
		void drawCell(const TCoordCell& cell, const Board& board, bool ingame=true);
		void drawBoard(const Board& board);
		void showCurrentPlayer(TPlayer player, int starters, const Board& board);
		void showWinner(TPlayer player);
		void toggleHelp(const Board& board);
		void nextHelpPage();
		void getTouchedCell(TCoordCell *cell, const Board& board);

	private:
		int bg[3];
		u16* mapMem[3];
		PrintConsole console;
		Audio *audio;
		bool help;
		int helpPage;
		TPlayer curPlayer;
		int curStarters;
		int cellCount;

		void drawPetalCell(const TCoordCell& cell, const Board& board);
		bool isPetalCell(int x, int y, const Board& board);
		static u16 playerToColor(TPlayer player);
};

#endif /* _UI_H */
