/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <nds.h>

#include "game.h"
#include "audio.h"
#include "options_ui.h"

int main() {
	Audio *audio = new Audio();
	UI *ui = new UI(audio);
	Options *options = new Options();
	OptionsUI optionsUI(*options, audio);
	Game *game = new Game(ui, options);

	TCoordCell cell;
	int pressed, held;

	srand(time(NULL));

	while (1) {
		scanKeys();
		pressed = keysDown();
		held = keysHeld();

		// START -> new game		
		if (pressed & KEY_START) {
			game->~Game();
			optionsUI.update(options);
			game = new Game(ui, options);
		}
		// SELECT -> show help / pause
		else if (pressed & KEY_SELECT) {
			game->toggleHelp();
		}
		// A -> next help page
		else if (pressed & KEY_A) {
			ui->nextHelpPage();
		}
		// D-PAD -> change options
		else if (pressed & (KEY_LEFT | KEY_RIGHT | KEY_UP | KEY_DOWN))
			optionsUI.navigate(pressed);
		// TOUCHSCREEN -> turn
		else if (game->humansTurn()) { 
			if (pressed & KEY_TOUCH) {
				ui->getTouchedCell(&cell, game->getBoard());
				game->turn(cell);
			}
		}
		// else -> AI turn
    else
			game->startAI();

		swiWaitForVBlank();
	}

	return 0;

}
